"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const mysql = require("mysql2/promise");
const tscommons_core_1 = require("tscommons-core");
const nodecommons_database_1 = require("nodecommons-database");
const nodecommons_database_2 = require("nodecommons-database");
class CommonsMysqlService extends nodecommons_database_2.CommonsSqlDatabaseService {
    constructor(credentials, pool = false, poolSize = 5, queueLimit = 5) {
        super(nodecommons_database_1.ECommonsDatabaseEngine.MYSQL, credentials);
        this.pool = pool;
        this.poolSize = poolSize;
        this.queueLimit = queueLimit;
    }
    static toUrl(credentials) {
        return `mysql://${credentials.user}:${credentials.password}@${credentials.host || 'localhost'}:${credentials.port || 5432}/${credentials.name}`;
    }
    connect() {
        return __awaiter(this, void 0, void 0, function* () {
            const internalCredentials = {
                host: this.credentials.host,
                user: this.credentials.user,
                password: this.credentials.password,
                database: this.credentials.name
            };
            if (this.pool) {
                internalCredentials['waitForConnections'] = true;
                internalCredentials['connectionLimit'] = this.poolSize;
                internalCredentials['queueLimit'] = this.queueLimit;
                internalCredentials['namedPlaceholders'] = true;
                this.database = mysql.createPool(internalCredentials);
            }
            else {
                this.database = yield mysql.createConnection(internalCredentials);
                this.database.config.namedPlaceholders = true;
            }
            if (this.database === undefined)
                throw new Error('Unable to create a single or pool connection database');
        });
    }
    doesTableExist(name) {
        return __awaiter(this, void 0, void 0, function* () {
            return (yield this.internalSelect(`SHOW TABLES LIKE ${this.table(name)}`)).length > 0;
        });
    }
    internalSelect(sql, params) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.database)
                throw new Error('No connection to database');
            try {
                let rows = [];
                if (params) {
                    const internal = yield this.database.execute(sql, params);
                    rows = internal[0];
                }
                else {
                    const internal = yield this.database.execute(sql);
                    rows = internal[0];
                }
                // get rid of the BinaryRow objects and turn them back into normal objects
                return (rows
                    .map((row) => tscommons_core_1.CommonsObject.mapObject(row, (value) => value)));
            }
            catch (e) {
                throw e;
            }
        });
    }
    internalNone(sql, params) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.database)
                throw new Error('No connection to database');
            try {
                if (params) {
                    yield this.database.execute(sql, params);
                }
                else {
                    yield this.database.execute(sql);
                }
            }
            catch (e) {
                console.log(e);
                throw e;
            }
        });
    }
    internalTransactionBegin() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.database)
                throw new Error('No connection to database');
            // not implemented yet
        });
    }
    internalTransactionCommit() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.database)
                throw new Error('No connection to database');
            // not implemented yet
        });
    }
    internalTransactionRollback() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.database)
                throw new Error('No connection to database');
            // not implemented yet
        });
    }
}
exports.CommonsMysqlService = CommonsMysqlService;
//# sourceMappingURL=commons-mysql.service.js.map